#ifndef Robust_
#define Robust_ 

#include "mies/SerialES/global.h"
#include "mies/SerialES/MixedIntegerES.h"

class Robust 
{
  public:  
    Robust(unsigned sampleSetSize_);

    virtual ~Robust() { deallocateSampleSet(); }
    
    virtual vector<double>* getF() = 0;
  protected:
    double** sampleSet;
    unsigned sampleSetSize;
    int numF;

    void allocateSampleSet(int numNoise);
    void deallocateSampleSet();

    /** \brief  Generates sampleSetSize Monte Carlo samples, which are noise vectors consisting of:
     *          numNoiseNorm normal noise elements,
     *          followed by numNoiseUnif uniform noise elements */
    void generateSampleSet_MC(int numNoise, int* numNoiseNorm, int* numNoiseUnif);

    /** \brief  Generates sampleSetSize Latin Hypercube samples, which are noise vectors consisting of:
     *          numNoiseNorm normal noise elements,
     *          followed by numNoiseUnif uniform noise elements */
    void generateSampleSet_LH(int numNoise, int* numNoiseNorm, int* numNoiseUnif);

    virtual void robustSimulate(Individual* I, int* xnom, int numVar) = 0;
    virtual int* getNumNoiseNorm() = 0; 
    virtual int* getNumNoiseUnif() = 0;
    
    virtual void simulate(Individual* I, int* xnom, int numVar, double* noise) = 0; 
};

#endif
