#ifndef WDNMIES_
#define WDNMIES_

#include "WDN.h"

#ifdef NSGAII
#include "mies/SerialES/NSGAII_MIES.h"
class WDN_MIES: public NSGAII_MIES, public WDN
#else
#include "mies/SerialES/SMSEMOA_Steady3D_MIES.h"
class WDN_MIES: public SMSEMOA_Steady3D_MIES, public WDN
#endif       
{
  public:
    WDN_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
	      unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
	        vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
	          vector<int>& lBound_d_, vector<int>& uBound_d_, double prob_min_, double prob_max_,
	            bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
	              bool selfAdaptation_, vector<int>& direction_, 
	                unsigned evaluations_, vector<double>& optimalF_, 
                          double initialSigma_r_, double initialSigma_z_, double initialProb_,
	         	    ifstream* initialPopFile_, int randomSeed_, bool feedback_, bool snapShots_, 
			      unsigned timeOutValue_,
                                string simInputFileName_, string simOutputFileName_,
			          unsigned selectDimension_, vector<unsigned>& selectFunction_,
				    bool excludeParent_,
				      double cvDemand_, double cvRoughness_, unsigned sampleSetSize_, bool LH_ = true) 
      :
#ifdef SMSEMOA3D
       SMSEMOA_Steady3D_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		              mu_, rho_, kappa_, lambda_,  
		                lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		                  lBound_d_, uBound_d_, prob_min_, prob_max_,
			            disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			              selfAdaptation_, direction_, 
			                evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
				          initialPopFile_, randomSeed_, feedback_, timeOutValue_,
			                    selectDimension_, selectFunction_,
				              excludeParent_),
#endif
       NSGAII_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		    mu_, rho_, kappa_, 
#ifdef LAMBDA
                                       lambda_, // Use (mu + lambda)-selection
#else
                                       mu_,     // Use (mu + mu)-selection, like prescribed in Deb, 2000 - NSGA-II   		   
#endif
		      lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		        lBound_d_, uBound_d_, prob_min_, prob_max_,
		          disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
		            selfAdaptation_, direction_, 
		              evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
		                initialPopFile_, randomSeed_, feedback_, timeOutValue_,
		                  selectDimension_, selectFunction_),
       WDN(simInputFileName_, simOutputFileName_,  cvDemand_, cvRoughness_, sampleSetSize_, LH_) 
       {  
         snapShots = snapShots_;

	 if (evaluations > 0)
	 {
#ifdef ROBUST
           generations = (evaluations - ((initialPopFile == NULL) ? mu * sampleSetSize : 0)) / (lambda * sampleSetSize); 
#endif
	 }
	 else
	 {   
	   cerr << "Supplied value for mumber of evaluations is " << evaluations << ", but should be larger than 0! Exiting." << endl;
           exit(1);
         }

         if (isParallelExp())
         { 
           /* Enable selection of zero diameter */ 
	   if (n_d == 0) // Added for integer-based optimization
	   {
             for (unsigned i = 0; i < n_z; i++)
               uBound_z[i]++;
	   }
           else
	   {	   
             for (unsigned i = 0; i < n_d; i++)
               uBound_d[i]++;
	   }
	 }
       }

    ~WDN_MIES() {}

  private: 
    bool snapShots;

    /** \brief  Initialize Individuals in population P with uniform randomly 
     *          chosen values within various variable and parameter domains 
     *	        or read previously generated population in from file 
     *          [and re-evaluate in case of robustness evaluation, then writePop() and exit] */
    void initialize(); 

    //! Determine fitness values for population P or O 
    void evaluate();

    //! Run simulator using the objective variables contained in Individual I 
    void simulate(Individual* I);

    //! Create lambda offspring
    void recombineMutate();

    //! Write current population P to file
    void writePop(time_t id);

    //! Output stepsizes 
    void writeStep(time_t id);

    //! Log time that was needed to complete generation
    void writeLog(time_t elapsed, time_t id);
};

#endif
