#ifndef APRIORI_MIES
#define APRIORI_MIES

#include <map>

#ifdef PARALLEL
#include "../ParallelES/MixedIntegerES_P.h"
/** \brief  Abstract class */
class APriori_MIES:public MixedIntegerES_P
{
  public:
    APriori_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		  unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		    vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		      vector<int>& lBound_d_, vector<int>& uBound_d_, double prob_min_, double prob_max_,
		        bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		          bool selfAdaptation_, vector<int>& direction_, 
		            unsigned evaluations_, vector<double>& optimalF_, 			      
			      double initialSigma_r_, double initialSigma_z_, double initialProb_,
			        ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			          unsigned timeOutVal_,
		    	            unsigned selectDimension_, vector<unsigned>& selectFunction_) 
      :MixedIntegerES_P(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		         mu_, rho_, kappa_, lambda_,  
		           lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		             lBound_d_, uBound_d_, prob_min_, prob_max_,
			       disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			         selfAdaptation_, direction_, 
			           evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
				     initialPopFile_, randomSeed_, feedback_, timeOutVal_) 
	                             { selectDimension = selectDimension_;
	                               selectFunction = selectFunction_;   
                                       selectFunctionActual = (selectDimension == 1) ? selectFunction[0] : n_f; }
#else
#include "MixedIntegerES.h"
/** \brief  Abstract class */
class APriori_MIES:public MixedIntegerES
{
  public:
    APriori_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		  unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		    vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		      vector<int>& lBound_d_, vector<int>& uBound_d_, double prob_min_, double prob_max_,
		        bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		          bool selfAdaptation_, vector<int>& direction_, 
		            unsigned evaluations_, vector<double>& optimalF_, 
                              double initialSigma_r_, double initialSigma_z_, double initialProb_,
			        ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			          unsigned timeOutVal_,
		    	            unsigned selectDimension_, vector<unsigned>& selectFunction_) 
      :MixedIntegerES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		       mu_, rho_, kappa_, lambda_,  
		         lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		           lBound_d_, uBound_d_, prob_min_, prob_max_,
			     disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			       selfAdaptation_, direction_, 
			         evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
				   initialPopFile_, randomSeed_, feedback_) 
	                           { selectDimension = selectDimension_;
	                             selectFunction = selectFunction_;   
                                     selectFunctionActual = (selectDimension == 1) ? selectFunction[0] : n_f; }
#endif
    virtual ~APriori_MIES() {}

  protected:
    /** \brief  Number of fitness functions used for selection */
    unsigned selectDimension;     

    /** \brief  Ids of the fitness functions used for selection */
    vector<unsigned> selectFunction; 

    /** \brief  selectFunction[0] if selectDimension = 1, else 'points' to the sum of the selectFunction values 
      *         (at pos n_f in vector F[] of each Individual) 
      */
    unsigned selectFunctionActual;   

    /** \brief  Run simulator using the objective variables contained in Individual I */
    virtual void simulate(Individual* I) = 0;

    /** \brief  Write current population P to file */
    virtual void writePop(time_t id) = 0;

    /** \brief  Select mu Individuals from O and optionally P to form new population P */
    void select();

    /** \brief  Sort Individuals on objective, in descending order */
    void quickSort(vector<int>& indices, int top, int bottom, vector<Individual*>* Q, int objective);
    int partition(vector<int>& indices, int top, int bottom, vector<Individual*>* Q, int objective);

    /** \brief  Determine bestF value for current population */
    void determineBestF(); 

    /** \brief  Log time that was needed to complete generation */
    virtual void writeLog(time_t elapsed, time_t id) = 0;

    /** \brief  In case of convergence study (i.e., with known optima supplied) determine whether optima have been reached */
    bool optimumReached(); 
};

#endif
