#ifndef NSGA_MIES
#define NSGA_MIES

#include "NSGA_II.h"

#ifdef PARALLEL 
#include "../ParallelES/MixedIntegerES_P.h"
/** \brief  Abstract class */
class NSGAII_MIES:public MixedIntegerES_P, virtual public NSGA_II 
{
  public:
    NSGAII_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		 unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		   vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		     vector<int> lBound_d_, vector<int> uBound_d_, double prob_min_, double prob_max_,
		       bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		         bool selfAdaptation_, vector<int>& direction_, 
		           unsigned evaluations_, vector<double>& optimalF_, 
			     double initialSigma_r_, double initialSigma_z_, double initialProb_,
			       ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			         unsigned timeOutVal_,
		    	           unsigned selectDimension_, vector<unsigned>& selectFunction_) 
      :MixedIntegerES_P(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		         mu_, rho_, kappa_, lambda_,  
		           lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		             lBound_d_, uBound_d_, prob_min_, prob_max_,
			       disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			         selfAdaptation_, direction_, 
			           evaluations_, optimalF_, 
				     initialSigma_r_, initialSigma_z_, initialProb_,
				       initialPopFile_, randomSeed_, feedback_, timeOutVal_) 
	                             { 
#ifdef DEBUG
                                       cout << "NSGAII_MIES::NSGAII_MIES()" << endl;
#endif	
			               selectDimension = selectDimension_;
				       selectFunction = selectFunction_;   
				     }
#else
#include "MixedIntegerES.h"
/** \brief  Abstract class */
class NSGAII_MIES:public MixedIntegerES, virtual public NSGA_II
{
  public:
    NSGAII_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		 unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		   vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		     vector<int> lBound_d_, vector<int> uBound_d_, double prob_min_, double prob_max_,
		       bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		         bool selfAdaptation_, vector<int>& direction_, 
		           unsigned evaluations_, vector<double>& optimalF_, 
			     double initialSigma_r_, double initialSigma_z_, double initialProb_,
			       ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			         unsigned timeOutVal_,
		    	           unsigned selectDimension_, vector<unsigned>& selectFunction_) 
      :MixedIntegerES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		       mu_, rho_, kappa_, lambda_,  
		         lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		           lBound_d_, uBound_d_, prob_min_, prob_max_,
			     disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			       selfAdaptation_, direction_, 
			         evaluations_, optimalF_, 
				   initialSigma_r_, initialSigma_z_, initialProb_,
				     initialPopFile_, randomSeed_, feedback_) 
	                             { 
#ifdef DEBUG
                                       cout << "NSGAII_MIES::NSGAII_MIES()" << endl;
#endif	   
			               selectDimension = selectDimension_;
				       selectFunction = selectFunction_;   
				     }
#endif
    virtual ~NSGAII_MIES() {}

  protected:
    /** \brief  Run simulator using the objective variables contained in Individual I */
    virtual void simulate(Individual* I) = 0; 

    /** \brief  Write current population P to file */
    virtual void writePop(time_t id) = 0;

    /** \brief  Log time that was needed to complete generation */
    virtual void writeLog(time_t elapsed, time_t id) = 0;

    /** \brief  Select mu Individuals from O and optionally P to form new 
     *          population P, using multiobjective selection scheme as proposed 
     *          for NSGA-II by Deb, 2000 */
    void select() { NSGA_II::select(P, O, mu, kappa, lambda, direction, bestF); } 

    /** \brief  Determine domination relationship between A and B, or determine best or worst values */
    MOComparison compare(Individual* A, Individual* B, vector<double>& bestWorstF, bool determineValues, bool worst)
                         { return NSGA_II::compare(A, B, direction, bestWorstF, determineValues, worst); }	                              

    /** \brief  Determine bestF value(s) for current population; only necessary for initial generation since otherwise taken care of in select() */
    void determineBestF();
    
    /** \brief  In case of convergence study (i.e., with known optima supplied) determine whether optima have been reached */
    bool optimumReached(); 
};

#endif
