/* This file contains the variable and function declarations */

# ifndef _GLOBAL_H_
# define _GLOBAL_H_

/* inserted: ereehuis */
# ifdef __cplusplus
# define EXTERN_C extern "C"
# else
# define EXTERN_C
# endif

# define INF   1.0e14
# define EPS   1.0e-14
# define EULER 2.71828182845905
# define PI    3.14159265358979

typedef struct
{
    int rank;
    double constr_violation;
    double *xreal;
    int **gene;
    double *xbin;
    int *xnom; /* inserted: ereehuis */
    double *obj;
    double *constr;
    double crowd_dist;
}
individual;

typedef struct
{
    individual *ind;
}
population;

typedef struct lists
{
    int index;
    struct lists *parent;
    struct lists *child;
}
list;

extern int nreal;
extern int nbin;
extern int nnom; /* inserted: ereehuis */
extern int nobj;
extern int *obj_direction; /* inserted: ereehuis */
extern int ncon;
extern int popsize;
extern double pcross_real;
extern double pcross_bin;
extern double pcross_nom; /* inserted: ereehuis */
extern double pmut_real;
extern double pmut_bin;
extern double pmut_nom; /* inserted: ereehuis */
extern double eta_c;
extern double eta_m;
extern int ngen;
extern int nbinmut;
extern int nrealmut;
extern int nnommut; /* inserted: ereehuis */
extern int nbincross;
extern int nrealcross;
extern int nnomcross; /* inserted: ereehuis */
extern int *nbits;
extern double *min_realvar;
extern double *max_realvar;
extern double *min_binvar;
extern double *max_binvar;
extern int *min_nomvar; /* inserted: ereehuis */
extern int *max_nomvar; /* inserted: ereehuis */
extern int bitlength;
extern double *nadir; /* inserted: ereehuis */
extern double *ideal; /* inserted: ereehuis */
extern int fixedNadirIdeal; /* inserted: ereehuis */ 
extern void (*test_problem)(double *xreal, double *xbin, int **gene, int *xnom, double *obj, double *constr); /* C-style function pointer,
                                                                                                               * inserted: ereehuis */
void allocate_memory_pop (population *pop, int size);
void allocate_memory_ind (individual *ind);
void deallocate_memory_pop (population *pop, int size);
void deallocate_memory_ind (individual *ind);

double maximum (double a, double b);
double minimum (double a, double b);

void crossover (individual *parent1, individual *parent2, individual *child1, individual *child2);
void realcross (individual *parent1, individual *parent2, individual *child1, individual *child2);
void bincross (individual *parent1, individual *parent2, individual *child1, individual *child2);
void nomcross (individual *parent1, individual *parent2, individual *child1, individual *child2); /* inserted: ereehuis */

void assign_crowding_distance_list (population *pop, list *lst, int front_size);
void assign_crowding_distance_indices (population *pop, int c1, int c2);
void assign_crowding_distance (population *pop, int *dist, int **obj_array, int front_size);

void decode_pop (population *pop);
void decode_ind (individual *ind);
void encode_ind (individual *ind); /* inserted: ereehuis */

int check_dominance (individual *a, individual *b);

void evaluate_pop (population *pop);
void evaluate_ind (individual *ind);

void fill_nondominated_sort (population *mixed_pop, population *new_pop);
void crowding_fill (population *mixed_pop, population *new_pop, int count, int front_size, list *cur);

void initialize_pop (population *pop);
void initialize_ind (individual *ind);

void insert (list *node, int x);
list* del (list *node);

void merge(population *pop1, population *pop2, population *pop3);
void copy_ind (individual *ind1, individual *ind2);

void mutation_pop (population *pop);
void mutation_ind (individual *ind);
void bin_mutate_ind (individual *ind);
void real_mutate_ind (individual *ind);
void nom_mutate_ind (individual *ind); /* inserted: ereehuis */

/* void test_problem (double *xreal, double *xbin, int **gene, double *obj, double *constr); removed: ereehuis */

void assign_rank_and_crowding_distance (population *new_pop);

void report_pop (population *pop, FILE *fpt);
void report_feasible (population *pop, FILE *fpt);
void report_ind (individual *ind, FILE *fpt);

void quicksort_front_obj(population *pop, int objcount, int obj_array[], int obj_array_size);
void q_sort_front_obj(population *pop, int objcount, int obj_array[], int left, int right);
void quicksort_dist(population *pop, int *dist, int front_size);
void q_sort_dist(population *pop, int *dist, int left, int right);

void selection (population *old_pop, population *new_pop);
individual* tournament (individual *ind1, individual *ind2);

EXTERN_C void setNadirIdeal(double *nadir, double *ideal); /* inserted: ereehuis */
EXTERN_C void generate(population *pop, individual *ind); /* inserted: ereehuis */
EXTERN_C void reduce(population *pop, individual *ind); /* inserted: ereehuis */
# endif
