#!/bin/csh -f

if ($# < 3) then
  echo "Usage: $0 N|I WDNproblem *convData.txt [*convData.txt ...]"
  exit
endif

set plotDir     = "plot"
set ghostScript = "gs"
set gnuplot     = "gnuplot"

$gnuplot << EOF
set term postscript landscape color enhanced dashed linewidth 1 "Arial" 20

set output "$plotDir/convplot.ps"

type = "$1"

set xrange [*:*]
set yrange [*:*]

set xtics auto
set ytics auto
set format y "%4.2f"

set grid xtics ytics lt 0 lw 1 lc rgb "#C3C3C3

set xlabel "{/Arial-Bold evaluations}"
set ylabel "{/Arial-Bold hypervolume}"

set size 1.055,1.18
set origin -0.02,-0.12

set lmargin 6
set rmargin 3

set key spacing 5
set key right bottom
set style line 1 lt -1 lw 1
set key box linestyle 1

if (type eq "N") plot '$3' w l lw 3 title "NSGA-II", '$4' w l lw 3 title "    SMS-EMOA"; else plot '$3' w l lw 3 title "NSGA-II-sa-sp-{/Symbol-Oblique=22 l}", '$4' w l lw 3 title "SMS-EMOA-sa";
EOF

"$ghostScript" -dSAFER -dBATCH -dNOPAUSE -sDEVICE=pdfwrite -sOutputFile=$plotDir/convplot$2.pdf $plotDir/convplot.ps

rm $plotDir/convplot.ps
