#!/usr/bin/perl

die "Usage: $0 N|R|I [MC|c|cd|cp|ar|ag] (N: normal hv calc, R: robust hv calc, I: normal hv calc integer runs; MC: MC integration, c: count only, cd: convergence data, cp: convergence plots, ar: att. surf. range, ag: att. surf. generate)\n" if ($#ARGV < 0);

use POSIX qw(ceil);

@WDNproblems   = ("TwoLoop"); # "Hanoi", "NewYorkCity");
$skip          = 0;
$doMonteCarlo  = 0;
$countOnly     = 0;
$convData      = 0;
$convPlots     = 0;
$attRange      = 0;
$attGenerate   = 0;
$testrunsDir   = "testruns";
$plotDir       = "plot";
$normScript    = "./normalize.pl";
$hvScript      = "./hypervolume.csh";
$bpScript      = "$plotDir/boxplot.csh";
$attScript     = "$plotDir/attain.csh";
$stepScript    = "$plotDir/stepsizes.csh";
$cpScript      = "$plotDir/convplot.csh";
$normFile      = "normalized.out";
$origFile      = "original.out";
$sleepInterval = 5;

if ($ARGV[0] eq "I")
{
  $testrunsDir = "$testrunsDir\_integer";
}

$resultsFile  = "$testrunsDir/results";
$allRunsFile  = "$testrunsDir/allRuns";
$elapsedFile  = "$testrunsDir/elapsed";

if ($#ARGV == 1)
{
  if ($ARGV[1] eq "MC")
  {
    $doMonteCarlo = 1;
  }    
  elsif ($ARGV[1] eq "c")
  {
    $countOnly = 1;
    $skip = 1;
  }
  elsif ($ARGV[1] eq "cd")
  {
    $convData = 1;
    $skip = 1;

    $mu = 40;
    $numEvals = 20000;
  }
  elsif ($ARGV[1] eq "cp")
  {
    $convPlots = 1;
    $skip = 1;    
  }
  elsif ($ARGV[1] eq "ar")
  {
    $attRange = 1;
    $skip = 1;

    system "g++ $plotDir/plot-attainments2.cc -o $plotDir/plot-att -lm -O3";
  }
  elsif ($ARGV[1] eq "ag")
  {
    $attGenerate = 1;
    $skip = 1;
  }
} 

if ($ARGV[0] eq "N")
{
  @MOEAs = ("NSGAII", "SMSEMOA");
      # "NSGAII_sa_sp_lambda", "SMSEMOA_sa",
	    #"NSGAII_sa", "NSGAII_sa_sp",
	    #"SMSEMOA_dp", "SMSEMOA_sa_dp",
	    #"NSGAII_sa_sp_lambda_SEM", "SMSEMOA_sa_SEM", 
	    #"NSGAII_sa_sp_lambda_MEM", "SMSEMOA_sa_MEM");
}
elsif ($ARGV[0] eq "R")
{
  $sampleSetSize = 2000;

  @MOEAs = ("NSGAII", "SMSEMOA", 
	    "NSGAII_sa_sp_lambda", "SMSEMOA_sa", 
            "NSGAII_sa_sp_lambda_SEM", "SMSEMOA_sa_SEM", 
	    "NSGAII_sa_sp_lambda_MEM", "SMSEMOA_sa_MEM");

  $resultsFile = $resultsFile . "_robust";
  $allRunsFile = $allRunsFile . "_robust";
  $elapsedFile = $elapsedFile . "_robust";

  if ($doMonteCarlo == 1)
  {
    system "make clean";
    system "make robeval=1";
    system "sleep $sleepInterval"; # Omit "Device or resource busy" message
    print "\n";
  }
}
elsif ($ARGV[0] eq "I")
{
  @MOEAs = ("NSGAII_sa_sp_lambda", "SMSEMOA_sa"); 
	    #"NSGAII_sa", "NSGAII_sa_sp", 
	    #"SMSEMOA_sa_dp");
}

$resultsFile = $resultsFile . ".txt";
$allRunsFile = $allRunsFile . ".txt";
$elapsedFile = $elapsedFile . ".txt";

if ($countOnly == 1)
{
  print "!! count only + attData generation !!\n\n";
}
elsif ($convData == 1)
{
  print "!! convergence plots data calculation !!\n\n";
}
elsif ($convPlots == 1)
{
  print "!! convergence plots generation !!\n\n";
}
elsif ($attRange == 1)
{
  print "!! attData range determination !!\n\n";
}
elsif ($attGenerate == 1)
{
  print "!! attainment surfaces generation !!\n\n";
}
else
{
  system "rm -f $resultsFile";
  open(RESF, ">>$resultsFile");

  system "rm -f $allRunsFile";
  open(ALLR, ">>$allRunsFile");

  system "rm -f $elapsedFile";
  open(ELAP, ">>$elapsedFile");
}

foreach $WDNproblem (@WDNproblems)
{
  $WDNproblem =~ /([THN])/x;	
  $currentWDN = $1;
  $currentMOEA = 1;

  if ($#ARGV == 0)
  {
    $currentBoxData = "$testrunsDir/bpData$currentWDN";
    
    if ($ARGV[0] eq "R")
    {
      $currentBoxData = $currentBoxData . "_robust";
    }
    $currentBoxData = $currentBoxData . ".txt";

    system "rm -f $currentBoxData";
    open(BOXD, ">>$currentBoxData");
  }
  elsif ($convPlots == 1)
  {
    $convFiles = "";	  
  }
  elsif ($attRange == 1)
  {
    @min = (10e+10, 10e+10, 10e+10);
    @max = (0, 0, 0);
  }

  if ($skip == 0)
  {
    print ELAP "$WDNproblem\n";
  }

  foreach $MOEA (@MOEAs)
  {  
    if ($ARGV[0] eq "R")
    {
      print "MC ($sampleSetSize): ";

      if ($skip == 0)
      {
        print RESF "MC ($sampleSetSize): ";
      } 
    }

    print "Processing $WDNproblem $MOEA test runs...\n";
    if ($skip == 0)
    { 
      print RESF "$WDNproblem $MOEA\n";
      print ALLR "$WDNproblem $MOEA\n";
    }
   
    $currentDir = "$testrunsDir/$WDNproblem\_$MOEA";
    $currentSummary = "$currentDir/summary";
    $currentAttData = "$currentDir/attData";

    if ($ARGV[0] eq "R")
    {
      $currentSummary = $currentSummary . "_robust";
      $currentAttData = $currentAttData . "_robust";
    }
    $currentSummary = $currentSummary . ".txt";
    $currentAttData = $currentAttData . ".txt";

    $currentConv = "$currentDir/convData.txt";

    if ($convPlots == 1)
    {
      $convFiles = $convFiles . " " . $currentConv;
      next;
    }
    elsif ($convData == 1)
    {    
      system "rm -f $currentConv";
      open(CONV, ">>$currentConv");

      $SSID = -1;
      for ($snapShot = $mu; $snapShot <= $numEvals; $snapShot += $mu)
      {
        $SSID++;
        opendir(DIR, $currentDir);
        @popFiles = grep(/RunID_[e\+\-\.\d]+_P_$snapShot\.txt$/,readdir(DIR));
        closedir(DIR);
     
        $numRuns = $#popFiles + 1;
	      if ($numRuns == 0)
	      {
	        next; # Skip if no files for this snapshot available
	      }

	      print "$snapShot ($numRuns): ";  
        $numVal = 0;
        $totalVal = 0;
        @HV = ();
        for $popFile (@popFiles)
	      {
          $popFile =~ /RunID_([e\+\-\.\d]+)_P_$snapShot\.txt$/x;

          if (-e "$currentDir/RunID_$1_convData.txt")
          {
            open(FILE, "<$currentDir/RunID_$1_convData.txt");
            @lines = <FILE>;
            close(FILE);
      
            $lines[$SSID] =~ /$snapShot\D+([e\+\-\.\d]+)/x;
            $HV[$numVal] = $1;
	          undef (@lines);
	        }
          else
	        {
	          $HV[$numVal] = `$hvScript $currentWDN $currentDir/$popFile`;
          }
	        $totalVal += $HV[$numVal];
	        $numVal++;

	        print "$numVal ";
	      }
	      $meanHV = $totalVal / $numVal;

        # Calculate hv standard deviation
        $stdHV = 0;
        for ($i = 0; $i < $numVal; $i++)
        {
          $stdHV += ($meanHV - $HV[$i])**2;
        }
        $stdHV = ($stdHV/($numVal+1))**0.5; # Sample variance

  	    print "\n";
        print CONV "$snapShot $meanHV $stdHV\n";

	      undef (@HV);
	      undef (@popFiles);
      }

      close(CONV);
      next;
    }
    elsif ($attRange == 1)
    {
      @lines = `$plotDir/plot-att $currentAttData -k 3 -mm -1 1 1 -r 1 -a 1 2>&1 1>/dev/null`;

      $i = 0;
      foreach $line (@lines)
      {
        if ($line =~ m/min =/)
        {
          $line =~ /\D+([e\+\-\.\d]+)\D+([e\+\-\.\d]+)/x;

          if ($1 < $min[$i])
	        {
            $min[$i] = $1;
	        }
          if ($2 > $max[$i])
	        {
            $max[$i] = $2;
	        }
          $i++;
        }
      }

      undef (@lines);
      next; 
    }
    elsif ($attGenerate == 1)
    {
      system "$attScript $currentWDN $currentAttData $WDNproblem\_$MOEA m"; # Only median attainment surface
      next;
    }

    # Generate stepsizes plot
    opendir(DIR, $currentDir);
    @stepFiles = grep(/RunID_[e\+\-\.\d]+_stepsizes\.txt$/,readdir(DIR));

    if ($#stepFiles >= 0)
    {
      $stepFiles[0] =~ /RunID_([e\+\-\.\d]+)/x;
      system "$stepScript $ARGV[0] $currentWDN $currentDir/$stepFiles[0] $1 > /dev/null";
      system "mv -f $plotDir/RunID*.pdf $currentDir >& /dev/null";
    }
    closedir(DIR);
    undef (@stepFiles);

    # Read in final population files of runs
    opendir(DIR, $currentDir);
    if ($ARGV[0] eq "R" && $doMonteCarlo == 0) # List results of previously run Monte Carlo integration
    {
      @popFiles = grep(/RunID_[e\+\-\.\d]+_P_robust\.txt$/,readdir(DIR));
    }
    else
    {
      @popFiles = grep(/RunID_[e\+\-\.\d]+_P\.txt$/,readdir(DIR));
    }
    closedir(DIR);
    $numRuns = $#popFiles + 1;

    # If no runs continue to next WDNproblem - MOEA combination
    if ($numRuns == 0)
    {
      print      "$numRuns runs\n"; 
      print RESF "$numRuns runs\n"; 
      next;
    }    
    
    system "rm -f $currentAttData";
    open(ATTD, ">>$currentAttData");

    if ($countOnly == 1)
    {
      open(SUMM, "<$currentSummary");
      @lines = <SUMM>;
      close(SUMM);

      if ($#lines == -1)
      {
	$processed = 0;
      }
      else
      {
	$processed = $#lines - (($ARGV[0] eq "R") ? 2 : 3);
      }
      undef (@lines);

      print "$numRuns runs, processed: $processed\n";
    }
    else
    {
      system "rm -f $currentSummary";
      open(SUMM, ">>$currentSummary");

      $numVal   = 0;
      $totalVal = 0;
      @HV = ();
    }

    # Calculate mean hv score and/or generate attainment surfaces plot input
    foreach $popFile (@popFiles) 
    {
      $popFile =~ /RunID_([e\+\-\.\d]+)/x;
      $runID = $1;

      if ($countOnly == 1)
      {
        system "rm -f $normFile"; 
        system "rm -f $origFile";
        system "$normScript $currentWDN $currentDir/$popFile"; 
      }
      else
      {
        if ($ARGV[0] eq "R") # Robust hv
        {
  	  if ($doMonteCarlo == 1) # Do Monte Carlo integration 
	  {
            if (!(-e "$currentDir/RunID_$runID_P_robust.txt"))
	    {
              $HV[$numVal] = `$hvScript $currentWDN $currentDir/$popFile $runID $sampleSetSize RunID_$runID_P_robust.txt`;
            }
	    else
	    {
  	      print "[RunID_$runID_P_robust.txt available]\n";	
              $HV[$numVal] = `$hvScript $currentWDN $currentDir/RunID_$runID_P_robust.txt`;
	    }
          }
	  else # Process results of previously run Monte Carlo integration
	  {
            $HV[$numVal] = `$hvScript $currentWDN $currentDir/$popFile`;
	  }
        }
        else # 'Normal' hv
        {  
          if (-e "$currentDir/RunID_$runID\_convData.txt")
          {
            open(FILE, "<$currentDir/RunID_$runID\_convData.txt");
            @lines = <FILE>;
            close(FILE);
      
            $lines[$#lines] =~ /[e\+\-\.\d]+\D+([e\+\-\.\d]+)/x;
            $HV[$numVal] = "$1\n";
	    undef (@lines);
	  }
	  else
	  {
            $HV[$numVal] = `$hvScript $currentWDN $currentDir/$popFile`;
          }
        }
        system "sleep $sleepInterval"; # Omit "Device or resource busy" message

        $HV[$numVal] =~ /([e\+\-\.\d]+)/x;
        $HV[$numVal] = $1;

        $totalVal += $HV[$numVal];
        $numVal++; 

        $timestamp = getTimestamp();
        print "" . (($numVal < 10) ? " " : "") . "$numVal [$timestamp] HV" . (($ARGV[0] eq "R") ? "[MC ($sampleSetSize)] " : " ") 
          . "RunID $runID:\t$HV[$numVal-1]\n";
        print SUMM (($numVal < 10) ? " " : "") . "$numVal [$timestamp] HV" . (($ARGV[0] eq "R") ? "[MC ($sampleSetSize)] " : " ") 
          . "RunID $runID:\t$HV[$numVal-1]\n";
      }

      # Generate attainment surfaces plot input  
      @lines = `cat original.out`;
      foreach $line (@lines)
      {
        print ATTD "$line";
      }
      print ATTD "\n";  
      undef (@lines);  
    }
    close(ATTD);
    undef (@popFiles);

    if ($countOnly == 1)
    {
      next;
    }
      
    if ($ARGV[0] eq "R" && $doMonteCarlo == 1)
    {
      system "mv RunID_* $currentDir";
    }
    $meanHV = $totalVal / $numVal;

    # Calculate hv standard deviation
    $stdHV = 0;
    for ($i = 0; $i < $numVal; $i++)
    {
      $stdHV += ($meanHV - $HV[$i])**2;
    }
    $stdHV = ($stdHV/($numVal+1))**0.5; # Sample variance

    # Calculate mean elapsed time
    opendir(DIR, $currentDir);
    @logFiles = grep(/log\.txt$/,readdir(DIR));
    closedir(DIR);

    $numVal   = 0;
    $totalVal = 0;
    @elapsed = ();
    foreach $logFile (@logFiles)
    {
      open(FILE, "<$currentDir/$logFile");
      @lines = <FILE>;
      close(FILE);
      
      if ($lines[$#lines] =~ m/Total optimization/)
      {
        $lines[$#lines] =~ /\s([e\+\-\.\d]+)/x;

	$elapsed[$numVal] = $1;
        $totalVal += $1;
        $numVal++; 
      }
      undef (@lines);
    }
    $meanElapsed = $totalVal / $numVal;

    # Calculate elapsed standard deviation
    $stdElapsed = 0;
    for ($i = 0; $i < $numVal; $i++)
    {
      $stdElapsed += ($meanElapsed - $elapsed[$i])**2;
    }
    $stdElapsed = ($stdElapsed/($numVal+1))**0.5; # Sample variance
    undef (@logFiles);
    undef (@elapsed);

    # Print results
    print SUMM "\n$numRuns runs\nmean HV " . (($ARGV[0] eq "R") ? "[MC ($sampleSetSize)]" : "") . ":\t$meanHV, std:\t$stdHV\n";  
    if ($ARGV[0] ne "R") 
    {
      print SUMM "mean elapsed:\t$meanElapsed, std:\t$stdElapsed";
    }
    close(SUMM);

    print      "$numRuns runs, mean HV: $meanHV, std: $stdHV, mean elapsed: $meanElapsed, std: $stdElapsed\n"; 
    print RESF "$numRuns runs, mean HV: $meanHV, std: $stdHV, mean elapsed: $meanElapsed, std: $stdElapsed\n";
    print ELAP "$MOEA\t$meanElapsed\t$stdElapsed\n";

    @HV = sort {$a <=> $b} @HV;
    for ($i = 0; $i < $numRuns; $i++)
    {
      print ALLR "$HV[$i]\n";
    }
    print ALLR "\n";

    # Generate box plot
    $min    = 0;
    $q1     = ceil($numRuns/4) - 1; 
    $median = ceil($numRuns/2) - 1;
    $q3     = $numRuns - $q1;
    $max    = $numRuns - 1; 
    
    print BOXD "$currentMOEA $HV[$min] $HV[$q1] $HV[$median] $HV[$q3] $HV[$max]\n";

    undef (@HV);
    $currentMOEA++;
  }

  if ($attRange == 1)
  {
    $currentBound = "$plotDir/bound" . $currentWDN . ".txt";
    system "rm -f $currentBound";
    open(BOUND, ">>$currentBound");

    print BOUND "$min[0] $min[1] $min[2]\n";
    print BOUND "$max[0] $max[1] $max[2]\n";

    close(BOUND);
  }

  if ($#ARGV == 0)
  {
    close(BOXD);
    system "$bpScript $ARGV[0] $currentWDN $currentBoxData > /dev/null";
  }

  if ($convPlots == 1)
  {  
    system "$cpScript $ARGV[0] $currentWDN " . $convFiles . " > /dev/null";  
  }

  print      "\n";
  print RESF "\n";
  print ALLR "\n";
  print ELAP "\n";
}

if ($countOnly == 1)
{
  print "!! count only + attData generation !!\n\n";
}
elsif ($convData == 1)
{
  print "!! convergence plots data calculation !!\n\n";
}
elsif ($convPlots == 1)
{
  print "!! convergence plots generation !!\n\n";
}
elsif ($attRange == 1)
{
  print "!! attData range determination !!\n\n";
}
elsif ($attGenerate == 1)
{
  print "!! attainment surfaces generation !!\n\n";
}
else
{
  close(RESF);   
  close(ALLR);   
  close(ELAP);   
}

system "mv -f $plotDir/*.pdf $plotDir/*.png $testrunsDir >& /dev/null";

#==============================================================================

sub getTimestamp()
{
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
 
  $year = sprintf("%.4u", $year+1900);
  $mon  = sprintf("%.2u", $mon+1);
  $mday = sprintf("%.2u", $mday);
  $hour = sprintf("%.2u", $hour);
  $min  = sprintf("%.2u", $min);

  return "$year-$mon-$mday $hour:$min";
}

#==============================================================================

